/**
* Creación de tablas y relaciones.
*
* Autor: Ángel Martínez de las Heras
*/

-- Creación de las tablas de la base de datos
CREATE TABLE Application (
    IDApplication INT PRIMARY KEY,
    name VARCHAR2(255) NOT NULL,
    isInProduction NUMBER(1) CHECK (isInProduction IN (0, 1)) NOT NULL,
    productionDate DATE,
    averageUsersNumber INT,
    technology VARCHAR2(255) NOT NULL,
    infraestructureType VARCHAR2(255) NOT NULL,
    criticality VARCHAR2(255) NOT NULL,
    geographicZoneID INT NOT NULL,
    technicalContact INT NOT NULL,
    businessContact INT NOT NULL
);

CREATE TABLE GeographicZone (
    IDZone INT PRIMARY KEY,
    name VARCHAR2(255) NOT NULL,
    description VARCHAR2(1000)
);

CREATE TABLE ApproverRole (
    IDRole INT PRIMARY KEY,
    name VARCHAR2(255) NOT NULL,
    priorityLevel NUMBER(3,0) CHECK (priorityLevel BETWEEN 1 AND 100),
    hasVote NUMBER(1) CHECK (hasVote IN (0, 1)) NOT NULL
);

CREATE TABLE Area (
    IDArea INT PRIMARY KEY,
    name VARCHAR2(255) NOT NULL
);

CREATE TABLE Approver (
    IDApprover INT PRIMARY KEY,
    name VARCHAR2(255) NOT NULL,
    contact VARCHAR2(255) NOT NULL,
    isAvailable NUMBER(1) CHECK (isAvailable IN (0, 1)) NOT NULL,
    approverRoleID INT NOT NULL,
    areaID INT NOT NULL,
    substituteID INT
);

CREATE TABLE Unavailability (
    IDUnavailability INT PRIMARY KEY,
    approverID INT NOT NULL,
    planned NUMBER(1) CHECK (planned IN (0, 1)) NOT NULL
);

CREATE TABLE Approval (
    approverID INT,
    changeID INT,
    approvalDate DATE NOT NULL,
    vote NUMBER(1) CHECK (vote IN (0, 1)),
    comments VARCHAR2(1000),
    PRIMARY KEY (approverID, changeID)
);

CREATE TABLE ApplicationApprovers (
    applicationID INT,
    approverID INT,
    PRIMARY KEY (applicationID, approverID)
);

CREATE TABLE ChangeCategory (
    IDCategory INT PRIMARY KEY,
    description VARCHAR2(1000) NOT NULL,
    importance NUMBER(2) NOT NULL,
    creationDate DATE NOT NULL,
    creationApprovalDate DATE NOT NULL,
    deleteApprovalDate DATE
);

CREATE TABLE Change (
    IDChange INT PRIMARY KEY,
    description VARCHAR2(1000) NOT NULL,
    requestDate DATE NOT NULL,
    status VARCHAR2(50) CHECK (status IN ('in_process', 'approved', 'rejected')) NOT NULL,
    changeCategoryID INT NOT NULL,
    applicationID INT NOT NULL
);

CREATE TABLE ChangeExecution (
    IDExecution INT PRIMARY KEY,
    executionDate DATE,
    result VARCHAR2(50) CHECK (result IN ('correct', 'incidents', 'rescheduled')),
    changeID INT NOT NULL
);

CREATE TABLE ActionPlan (
    IDActionPlan INT PRIMARY KEY,
    description VARCHAR2(1000) NOT NULL,
    isClosed NUMBER(1) CHECK (isClosed IN (0, 1)) NOT NULL,
    changeExecutionID INT NOT NULL
);

CREATE TABLE Action (
    IDAction INT PRIMARY KEY,
    description VARCHAR2(1000) NOT NULL,
    deliveryDate DATE NOT NULL,
    doneDate DATE,
    actionPlanID INT NOT NULL
);

CREATE TABLE ProcessAudit (
    IDAudit INT PRIMARY KEY,
    auditDate DATE NOT NULL,
    observations VARCHAR2(1000)
);

CREATE TABLE AuditComment (
    IDComment INT PRIMARY KEY,
    nonCompliances VARCHAR2(1000) NOT NULL,
    observations VARCHAR2(1000),
    auditID INT NOT NULL
);

CREATE TABLE LogProcedure (
    IDLog NUMBER PRIMARY KEY,
    logDateTime TIMESTAMP NOT NULL,
    logUser VARCHAR2(255) NOT NULL,
    procedureName VARCHAR2(255) NOT NULL,
    procedureParams VARCHAR2(1000),
    RSP VARCHAR2(1000) NOT NULL
);


-- Creación de repositorio estadístico en su tablespace
CREATE TABLE RE_Year (
    yearData INT PRIMARY KEY,
    changesInApproval INT,
    incorrectExecution INT,
    maxCorrectChangesTechnicalManager VARCHAR2(255),
    doneActionsInTimePercentaje NUMBER(5,2) CHECK (doneActionsInTimePercentaje BETWEEN 0 AND 100),
    GCABApprovedChanges INT,
    notApprovedChanges INT,
    openActionPlans INT,
    maxOpenChangesTechnicalManager VARCHAR2(255),
    mostReplacedApprover VARCHAR2(255)
) TABLESPACE RE_DATA;

CREATE TABLE RE_Geographic (
    geographicZone VARCHAR2(255) PRIMARY KEY,
    percentajeRescheduledChanges NUMBER(5,2) CHECK (percentajeRescheduledChanges BETWEEN 0 AND 100)
) TABLESPACE RE_DATA;

CREATE TABLE RE_LastSixMonths (
    lastDate DATE PRIMARY KEY,
    numNonCompliances INT
) TABLESPACE RE_DATA;


-- Creación de FK
ALTER TABLE Application ADD CONSTRAINT fk_application_geographiczone FOREIGN KEY (geographicZoneID) REFERENCES GeographicZone (IDZone);
ALTER TABLE Application ADD CONSTRAINT fk_application_technicalcontact FOREIGN KEY (technicalContact) REFERENCES Approver (IDApprover);
ALTER TABLE Application ADD CONSTRAINT fk_application_businesscontact FOREIGN KEY (businessContact) REFERENCES Approver (IDApprover);

ALTER TABLE Approver ADD CONSTRAINT fk_approver_approverrole FOREIGN KEY (approverRoleID) REFERENCES ApproverRole (IDRole);
ALTER TABLE Approver ADD CONSTRAINT fk_approver_area FOREIGN KEY (areaID) REFERENCES Area (IDArea);
ALTER TABLE Approver ADD CONSTRAINT fk_approver_substitute FOREIGN KEY (substituteID) REFERENCES Approver (IDApprover);

ALTER TABLE Unavailability ADD CONSTRAINT fk_unavailability_approver FOREIGN KEY (approverID) REFERENCES Approver (IDApprover);

ALTER TABLE Approval ADD CONSTRAINT fk_approval_approver FOREIGN KEY (approverID) REFERENCES Approver (IDApprover);
ALTER TABLE Approval ADD CONSTRAINT fk_approval_change FOREIGN KEY (changeID) REFERENCES Change (IDChange);

ALTER TABLE ApplicationApprovers ADD CONSTRAINT fk_applicationapprovers_application FOREIGN KEY (applicationID) REFERENCES Application (IDApplication);
ALTER TABLE ApplicationApprovers ADD CONSTRAINT fk_applicationapprovers_approver FOREIGN KEY (approverID) REFERENCES Approver (IDApprover);

ALTER TABLE Change ADD CONSTRAINT fk_change_changecategory FOREIGN KEY (changeCategoryID) REFERENCES ChangeCategory (IDCategory);
ALTER TABLE Change ADD CONSTRAINT fk_change_application FOREIGN KEY (applicationID) REFERENCES Application (IDApplication);

ALTER TABLE ChangeExecution ADD CONSTRAINT fk_changeexecution_change FOREIGN KEY (changeID) REFERENCES Change (IDChange);

ALTER TABLE ActionPlan ADD CONSTRAINT fk_actionplan_changeexecution FOREIGN KEY (changeExecutionID) REFERENCES ChangeExecution (IDExecution);

ALTER TABLE Action ADD CONSTRAINT fk_action_actionplan FOREIGN KEY (actionPlanID) REFERENCES ActionPlan (IDActionPlan);

ALTER TABLE AuditComment ADD CONSTRAINT fk_auditcomment_processaudit FOREIGN KEY (auditID) REFERENCES ProcessAudit (IDAudit);

