/**
* Creación de procedimientos ABM.
*
* Autor: Ángel Martínez de las Heras
*/




/**
 * Procedimiento addApplication
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'Application'.
 *   Registra cada operación en la tabla 'LogProcedure'.
 *   El campo 'isInProduction' siempre se establece en 0 y 'productionDate' siempre en NULL.
 * 
 * Parámetros:
 *   P_name IN Application.name%TYPE
 *     - Descripción: Nombre de la aplicación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_averageUsersNumber IN Application.averageUsersNumber%TYPE
 *     - Descripción: Número promedio de usuarios de la aplicación.
 *     - Tipo: Entero (INT).
 *     - Restricciones: Puede ser NULL.
 * 
 *   P_technology IN Application.technology%TYPE
 *     - Descripción: Tecnología utilizada por la aplicación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_infraestructureType IN Application.infraestructureType%TYPE
 *     - Descripción: Tipo de infraestructura de la aplicación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_criticality IN Application.criticality%TYPE
 *     - Descripción: Nivel de criticidad de la aplicación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_geographicZoneID IN Application.geographicZoneID%TYPE
 *     - Descripción: ID de la zona geográfica de la aplicación.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_technicalContact IN Application.technicalContact%TYPE
 *     - Descripción: ID del contacto técnico de la aplicación.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_businessContact IN Application.businessContact%TYPE
 *     - Descripción: ID del contacto de negocio de la aplicación.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'Application' con 'isInProduction' en 0 y 'productionDate' en NULL.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE addApplication(
    P_name IN Application.name%TYPE,
    P_averageUsersNumber IN Application.averageUsersNumber%TYPE,
    P_technology IN Application.technology%TYPE,
    P_infraestructureType IN Application.infraestructureType%TYPE,
    P_criticality IN Application.criticality%TYPE,
    P_geographicZoneID IN Application.geographicZoneID%TYPE,
    P_technicalContact IN Application.technicalContact%TYPE,
    P_businessContact IN Application.businessContact%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
BEGIN
    inputParams:= 'Name: '||P_name||' - AverageUsersNumber: '||P_averageUsersNumber||' - Technology: '||P_technology||' - InfrastructureType: '||P_infraestructureType||' - Criticality: '||P_criticality||' - GeographicZoneID: '||P_geographicZoneID||' - TechnicalContact: '||P_technicalContact||' - BusinessContact: '||P_businessContact;

    INSERT INTO Application (name, isInProduction, productionDate, averageUsersNumber, technology, infraestructureType, criticality, geographicZoneID, technicalContact, businessContact) 
        VALUES (P_name, 0, NULL, P_averageUsersNumber, P_technology, P_infraestructureType, P_criticality, P_geographicZoneID, P_technicalContact, P_businessContact);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addApplication', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addApplication', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento launchApplication
 * 
 * Descripción:
 *   Actualiza un registro en 'Application', estableciendo 'isInProduction' en 1 y 'productionDate' en la fecha actual.
 *   Verifica que 'P_applicationID' no sea NULL y exista en 'Application'.
 *   Registra la operación en 'LogProcedure'.
 * 
 * Parámetros:
 *   P_applicationID IN Application.IDApplication%TYPE
 *     - ID de la aplicación a actualizar.
 *     - No puede ser NULL, debe existir en 'Application'.
 * 
 *   RSP OUT VARCHAR2
 *     - Indica el resultado de la operación ('OK' para éxito, 'ERROR' para fallos).
 * 
 * Funcionamiento:
 *   1. Verifica 'P_applicationID'.
 *   2. Actualiza 'isInProduction' a 1 y 'productionDate' con fecha actual.
 *   3. Registra la operación en 'LogProcedure'.
 *   4. Maneja excepciones.
 * 
 * Excepciones:
 *   - NULL_VALUE_ERROR: Se lanza si 'P_applicationID' es NULL.
 *   - ID_NOT_FOUND_EXCEPTION: Se lanza si 'P_applicationID' no existe en 'Application'.
 *   - OTHERS: Captura otros errores.
 */

CREATE OR REPLACE PROCEDURE launchApplication(
    P_applicationID IN Application.IDApplication%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    NULL_VALUE_ERROR EXCEPTION;
    ID_NOT_FOUND_EXCEPTION EXCEPTION;
    applicationExists INT;

BEGIN
    inputParams:= 'ApplicationID: '||P_applicationID;

    IF P_applicationID IS NULL THEN
        RAISE NULL_VALUE_ERROR;
    END IF;

    SELECT COUNT(*) INTO applicationExists FROM Application WHERE IDApplication = P_applicationID;

    IF applicationExists = 0 THEN
        RAISE ID_NOT_FOUND_EXCEPTION;
    END IF;

    UPDATE Application 
    SET isInProduction = 1, 
        productionDate = SYSDATE
    WHERE IDApplication = P_applicationID;
    
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'launchApplication', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN NULL_VALUE_ERROR THEN
        RSP := 'ERROR - ApplicationID cannot be NULL';
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'launchApplication', inputParams, RSP);
        COMMIT;
    WHEN ID_NOT_FOUND_EXCEPTION THEN
        RSP := 'ERROR - ApplicationID does not exist';
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'launchApplication', inputParams, RSP);
        COMMIT;
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'launchApplication', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento updateApplication
 * 
 * Descripción:
 *   Este procedimiento se utiliza para actualizar un registro existente en la tabla 'Application'.
 *   Permite modificar los campos averageUsersNumber, technology, infraestructureType, 
 *   criticality, geographicZoneID, technicalContact y businessContact.
 *   Registra cada operación en la tabla 'LogProcedure'.
 * 
 * Parámetros:
 *   P_applicationID IN Application.IDApplication%TYPE
 *     - Descripción: ID de la aplicación a actualizar.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla 'Application'.
 *   P_averageUsersNumber IN Application.averageUsersNumber%TYPE (opcional)
 *   P_technology IN Application.technology%TYPE (opcional)
 *   P_infraestructureType IN Application.infraestructureType%TYPE (opcional)
 *   P_criticality IN Application.criticality%TYPE (opcional)
 *   P_geographicZoneID IN Application.geographicZoneID%TYPE (opcional)
 *   P_technicalContact IN Application.technicalContact%TYPE (opcional)
 *   P_businessContact IN Application.businessContact%TYPE (opcional)
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Actualiza los campos especificados de la aplicación en la tabla 'Application'.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE updateApplication(
    P_applicationID IN Application.IDApplication%TYPE,
    P_averageUsersNumber IN Application.averageUsersNumber%TYPE DEFAULT NULL,
    P_technology IN Application.technology%TYPE DEFAULT NULL,
    P_infraestructureType IN Application.infraestructureType%TYPE DEFAULT NULL,
    P_criticality IN Application.criticality%TYPE DEFAULT NULL,
    P_geographicZoneID IN Application.geographicZoneID%TYPE DEFAULT NULL,
    P_technicalContact IN Application.technicalContact%TYPE DEFAULT NULL,
    P_businessContact IN Application.businessContact%TYPE DEFAULT NULL,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);

BEGIN
    inputParams:= 'ApplicationID: '||P_applicationID||' - AverageUsersNumber: '||P_averageUsersNumber||' - Technology: '||P_technology||' - InfrastructureType: '||P_infraestructureType||' - Criticality: '||P_criticality||' - GeographicZoneID: '||P_geographicZoneID||' - TechnicalContact: '||P_technicalContact||' - BusinessContact: '||P_businessContact;

    UPDATE Application 
    SET averageUsersNumber = COALESCE(P_averageUsersNumber, averageUsersNumber),
        technology = COALESCE(P_technology, technology),
        infraestructureType = COALESCE(P_infraestructureType, infraestructureType),
        criticality = COALESCE(P_criticality, criticality),
        geographicZoneID = COALESCE(P_geographicZoneID, geographicZoneID),
        technicalContact = COALESCE(P_technicalContact, technicalContact),
        businessContact = COALESCE(P_businessContact, businessContact)
    WHERE IDApplication = P_applicationID;
    
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'updateApplication', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'updateApplication', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento endApplicationLifeCycle
 * 
 * Descripción:
 *   Este procedimiento se utiliza para marcar una aplicación como fuera de producción, 
 *   indicando el fin de su ciclo de vida en la tabla 'Application'.
 *   Registra cada operación en la tabla 'LogProcedure'.
 * 
 * Parámetros:
 *   P_applicationID IN Application.IDApplication%TYPE
 *     - Descripción: ID de la aplicación a actualizar.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla 'Application'.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Actualiza el campo 'isInProduction' a 0 para la aplicación especificada.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE endApplicationLifeCycle(
    P_applicationID IN Application.IDApplication%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);

BEGIN
    inputParams:= 'ApplicationID: '||P_applicationID;

    UPDATE Application 
    SET isInProduction = 0
    WHERE IDApplication = P_applicationID;
    
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'endApplicationLifeCycle', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'endApplicationLifeCycle', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addGeographicZone
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'GeographicZone'
 *   y un registro correspondiente en 'RE_Geographic' con un porcentaje de cambios reprogramados inicial en 0.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja cualquier error que ocurra durante la inserción.
 * 
 * Parámetros:
 *   P_name IN GeographicZone.name%TYPE
 *     - Descripción: Nombre de la zona geográfica.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_description IN GeographicZone.description%TYPE
 *     - Descripción: Descripción de la zona geográfica.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: Puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'GeographicZone'.
 *   2. Inserta un registro correspondiente en 'RE_Geographic' con 'percentajeRescheduledChanges' en 0.
 *   3. Registra la operación en la tabla 'LogProcedure'.
 *   4. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE addGeographicZone(
    P_name IN GeographicZone.name%TYPE,
    P_description IN GeographicZone.description%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
BEGIN
    inputParams:= 'Name: '||P_name||' - Description: '||P_description;

    INSERT INTO GeographicZone (name, description) VALUES (P_name, P_description);

    INSERT INTO RE_Geographic (geographicZone, percentajeRescheduledChanges) 
        VALUES (P_name, 0);

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addGeographicZone', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addGeographicZone', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addApproverRole
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'ApproverRole'.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja cualquier error que ocurra durante la inserción.
 * 
 * Parámetros:
 *   P_name IN ApproverRole.name%TYPE
 *     - Descripción: Nombre del rol del aprobador.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_priorityLevel IN ApproverRole.priorityLevel%TYPE
 *     - Descripción: Nivel de prioridad del rol.
 *     - Tipo: Número (NUMBER).
 *     - Restricciones: Debe estar entre 1 y 100 o NULL.
 * 
 *   P_hasVote IN ApproverRole.hasVote%TYPE
 *     - Descripción: Indica si el rol tiene derecho a voto.
 *     - Tipo: Número (NUMBER).
 *     - Restricciones: Debe ser 0 (no) o 1 (sí).
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta un nuevo registro en la tabla 'ApproverRole'.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE addApproverRole(
    P_name IN ApproverRole.name%TYPE,
    P_priorityLevel IN ApproverRole.priorityLevel%TYPE,
    P_hasVote IN ApproverRole.hasVote%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
BEGIN
    inputParams:= 'Name: '||P_name||' - PriorityLevel: '||P_priorityLevel||' - HasVote: '||P_hasVote;

    INSERT INTO ApproverRole (name, priorityLevel, hasVote) VALUES (P_name, P_priorityLevel, P_hasVote);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addApproverRole', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addApproverRole', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addArea
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'Area'.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja cualquier error que ocurra durante la inserción.
 * 
 * Parámetros:
 *   P_name IN Area.name%TYPE
 *     - Descripción: Nombre del área.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta un nuevo registro en la tabla 'Area'.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE addArea(
    P_name IN Area.name%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
BEGIN
    inputParams:= 'Name: '||P_name;

    INSERT INTO Area (name) VALUES (P_name);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addArea', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addArea', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addApprover
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'Approver'.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja cualquier error que ocurra durante la inserción.
 *   El campo 'isAvailable' siempre se establece en 1 y 'substituteID' siempre en NULL.
 * 
 * Parámetros:
 *   P_name IN Approver.name%TYPE
 *     - Descripción: Nombre del aprobador.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_contact IN Approver.contact%TYPE
 *     - Descripción: Información de contacto del aprobador.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_approverRoleID IN Approver.approverRoleID%TYPE
 *     - Descripción: ID del rol del aprobador.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla 'ApproverRole'.
 * 
 *   P_areaID IN Approver.areaID%TYPE
 *     - Descripción: ID del área del aprobador.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla 'Area'.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'Approver' con 'isAvailable' en 1 y 'substituteID' en NULL.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE addApprover(
    P_name IN Approver.name%TYPE,
    P_contact IN Approver.contact%TYPE,
    P_approverRoleID IN Approver.approverRoleID%TYPE,
    P_areaID IN Approver.areaID%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
BEGIN
    inputParams:= 'Name: '||P_name||' - Contact: '||P_contact||' - ApproverRoleID: '||P_approverRoleID||' - AreaID: '||P_areaID;

    INSERT INTO Approver (name, contact, isAvailable, approverRoleID, areaID, substituteID) 
        VALUES (P_name, P_contact, 1, P_approverRoleID, P_areaID, NULL);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addApprover', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addApprover', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento assignSubstituteToApprover
 * 
 * Descripción:
 *   Asigna un aprobador sustituto a un aprobador oficial en la tabla 'Approver'.
 *   Actualiza el campo 'substituteID' del aprobador oficial con el ID del sustituto.
 *   Verifica que el rol del aprobador oficial y del sustituto sean el mismo.
 *   Registra cada operación en la tabla 'LogProcedure'.
 * 
 * Parámetros:
 *   P_approverID IN Approver.IDApprover%TYPE
 *     - ID del aprobador oficial.
 *     - No puede ser NULL, debe existir en 'Approver'.
 * 
 *   P_substituteID IN Approver.IDApprover%TYPE
 *     - ID del aprobador sustituto.
 *     - No puede ser NULL, debe existir en 'Approver'.
 * 
 *   RSP OUT VARCHAR2
 *     - Indica el resultado de la operación ('OK' para éxito, 'ERROR' para fallos).
 * 
 * Funcionamiento:
 *   1. Verifica la igualdad de roles entre el aprobador oficial y el sustituto.
 *   2. Actualiza 'substituteID' en 'Approver'.
 *   3. Registra la operación en 'LogProcedure'.
 *   4. Maneja excepciones durante la ejecución.
 * 
 * Excepciones:
 *   - NULL_VALUE_ERROR: Se lanza cuando algún valor introducido es NULL.
 *   - ID_NOT_FOUND_EXCEPTION: Se lanza cuando un ID no existe en 'Approver'.
 *   - ROLE_MISMATCH_EXCEPTION: Se lanza cuando los roles del aprobador y sustituto no coinciden.
 *   - OTHERS: Captura otros errores.
 */
CREATE OR REPLACE PROCEDURE assignSubstituteToApprover(
    P_approverID IN Approver.IDApprover%TYPE,
    P_substituteID IN Approver.IDApprover%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    NULL_VALUE_ERROR EXCEPTION;
    ID_NOT_FOUND_EXCEPTION EXCEPTION;
    ROLE_MISMATCH_EXCEPTION EXCEPTION;
    approverRoleID INT;
    substituteRoleID INT;

BEGIN
    inputParams:= 'ApproverID: '||P_approverID||' - SubstituteID: '||P_substituteID;

    IF P_approverID IS NULL OR P_substituteID IS NULL THEN
        RAISE NULL_VALUE_ERROR;
    END IF;

    -- Verifica existencia de approver y sustituto
    SELECT approverRoleID INTO approverRoleID FROM Approver WHERE IDApprover = P_approverID;
    SELECT approverRoleID INTO substituteRoleID FROM Approver WHERE IDApprover = P_substituteID;

    IF approverRoleID IS NULL OR substituteRoleID IS NULL THEN
        RAISE ID_NOT_FOUND_EXCEPTION;
    ELSIF approverRoleID != substituteRoleID THEN
        RAISE ROLE_MISMATCH_EXCEPTION;
    END IF;

    UPDATE Approver 
    SET substituteID = P_substituteID
    WHERE IDApprover = P_approverID;
    
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'assignSubstituteToApprover', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN NULL_VALUE_ERROR THEN
        RSP := 'ERROR - ApproverID or SubstituteID cannot be NULL';
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'assignSubstituteToApprover', inputParams, RSP);
        COMMIT;
    WHEN ID_NOT_FOUND_EXCEPTION THEN
        RSP := 'ERROR - ApproverID or SubstituteID does not exist';
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'assignSubstituteToApprover', inputParams, RSP);
        COMMIT;
    WHEN ROLE_MISMATCH_EXCEPTION THEN
        RSP := 'ERROR - Approver and Substitute roles do not match';
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'assignSubstituteToApprover', inputParams, RSP);
        COMMIT;
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'assignSubstituteToApprover', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addUnavailability
 * 
 * Descripción:
 *   Este procedimiento registra la indisponibilidad de un approver y asigna autorización a su sustituto.
 *   Inserta un registro en 'Unavailability' y actualiza 'isAvailable' a 0 en 'Approver'.
 *   Además, añade registros en 'ApplicationApprovers' para el sustituto en las mismas aplicaciones que el approver oficial.
 *   Si la indisponibilidad es no planificada, actualiza el approver más sustituido en 'RE_Year'.
 * 
 * Parámetros:
 *   P_approverID IN Unavailability.approverID%TYPE
 *     - ID del aprobador.
 *     - No puede ser NULL.
 * 
 *   P_planned IN Unavailability.planned%TYPE
 *     - Indica si la indisponibilidad es planificada (1) o no (0).
 *     - No puede ser NULL, debe ser 0 o 1.
 * 
 *   RSP OUT VARCHAR2
 *     - Indica el resultado de la operación ('OK' para éxito, 'ERROR' para fallos).
 * 
 * Funcionamiento:
 *   1. Inserta un registro en 'Unavailability' y actualiza 'isAvailable' a 0 en 'Approver'.
 *   2. Añade autorización para el sustituto en 'ApplicationApprovers'.
 *   3. Actualiza el approver más sustituido en 'RE_Year' si es no planificado.
 *   4. Registra la operación en 'LogProcedure'.
 *   5. Maneja excepciones.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier error.
 */
CREATE OR REPLACE PROCEDURE addUnavailability(
    P_approverID IN Unavailability.approverID%TYPE,
    P_planned IN Unavailability.planned%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    currentYear INT;
    mostReplacedApproverID INT;
    mostReplacedName VARCHAR2(255);
    mostReplacedContact VARCHAR2(255);
    maxReplacements INT;
    substituteID INT;
BEGIN
    inputParams:= 'ApproverID: '||P_approverID||' - Planned: '||P_planned;

    INSERT INTO Unavailability (approverID, planned) VALUES (P_approverID, P_planned);

    SELECT substituteID INTO substituteID FROM Approver WHERE IDApprover = P_approverID;
    UPDATE Approver SET isAvailable = 0 WHERE IDApprover = P_approverID;

    -- Añadir registros en ApplicationApprovers para el sustituto
    IF substituteID IS NOT NULL THEN
        FOR r IN (SELECT applicationID FROM ApplicationApprovers WHERE approverID = P_approverID) LOOP
            INSERT INTO ApplicationApprovers (applicationID, approverID) VALUES (r.applicationID, substituteID);
        END LOOP;
    END IF;

    IF P_planned = 0 THEN
        SELECT EXTRACT(YEAR FROM SYSDATE) INTO currentYear FROM DUAL;

        -- Calcular el approver más sustituido por indisponibilidades no planificadas
        SELECT IDApprover, name, contact, COUNT(*) INTO mostReplacedApproverID, mostReplacedName, mostReplacedContact, maxReplacements
        FROM Unavailability
        JOIN Approver ON Unavailability.approverID = Approver.IDApprover
        WHERE planned = 0
        GROUP BY IDApprover, name, contact
        ORDER BY COUNT(*) DESC
        FETCH FIRST ROW ONLY;

        UPDATE RE_Year SET mostReplacedApprover = mostReplacedName || '-' || mostReplacedContact WHERE yearData = currentYear;
    END IF;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addUnavailability', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addUnavailability', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento setApproverAvailability
 * 
 * Descripción:
 *   Actualiza el estado de disponibilidad de un approver en 'Approver' a disponible (isAvailable = 1).
 *   Elimina las autorizaciones del sustituto en 'ApplicationApprovers' establecidas durante la indisponibilidad del approver.
 *   Registra la operación en 'LogProcedure'.
 * 
 * Parámetros:
 *   P_approverID IN Approver.IDApprover%TYPE
 *     - ID del aprobador.
 *     - No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Indica el resultado de la operación ('OK' para éxito, 'ERROR' para fallos).
 * 
 * Funcionamiento:
 *   1. Actualiza 'isAvailable' a 1 en 'Approver'.
 *   2. Elimina las autorizaciones del sustituto en 'ApplicationApprovers'.
 *   3. Registra la operación en 'LogProcedure'.
 *   4. Maneja excepciones.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier error.
 */
CREATE OR REPLACE PROCEDURE setApproverAvailability(
    P_approverID IN Approver.IDApprover%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    substituteID INT;
BEGIN
    inputParams:= 'ApproverID: '||P_approverID;

    SELECT substituteID INTO substituteID FROM Approver WHERE IDApprover = P_approverID;
    UPDATE Approver SET isAvailable = 1 WHERE IDApprover = P_approverID;

    -- Eliminar autorizaciones del sustituto
    IF substituteID IS NOT NULL THEN
        DELETE FROM ApplicationApprovers WHERE approverID = substituteID;
    END IF;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'setApproverAvailability', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'setApproverAvailability', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addApplicationApprover
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'ApplicationApprovers'.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja cualquier error que ocurra durante la inserción.
 * 
 * Parámetros:
 *   P_applicationID IN ApplicationApprovers.applicationID%TYPE
 *     - Descripción: ID de la aplicación.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_approverID IN ApplicationApprovers.approverID%TYPE
 *     - Descripción: ID del aprobador.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'ApplicationApprovers'.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE addApplicationApprover(
    P_applicationID IN ApplicationApprovers.applicationID%TYPE,
    P_approverID IN ApplicationApprovers.approverID%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
BEGIN
    inputParams:= 'ApplicationID: '||P_applicationID||' - ApproverID: '||P_approverID;

    INSERT INTO ApplicationApprovers (applicationID, approverID) VALUES (P_applicationID, P_approverID);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addApplicationApprover', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addApplicationApprover', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addChangeCategory
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'ChangeCategory'.
 *   Registra cada operación en la tabla 'LogProcedure'.
 *   'creationDate' siempre se establece en la fecha actual y 'deleteApprovalDate' siempre en NULL.
 * 
 * Parámetros:
 *   P_description IN ChangeCategory.description%TYPE
 *     - Descripción: Descripción de la categoría de cambio.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_importance IN ChangeCategory.importance%TYPE
 *     - Descripción: Importancia de la categoría de cambio.
 *     - Tipo: Número (NUMBER).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_creationApprovalDate IN ChangeCategory.creationApprovalDate%TYPE
 *     - Descripción: Fecha de aprobación de la creación de la categoría.
 *     - Tipo: Fecha (DATE).
 *     - Restricciones: No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'ChangeCategory' con 'creationDate' actual, 'creationApprovalDate' proporcionado y 'deleteApprovalDate' en NULL.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE addChangeCategory(
    P_description IN ChangeCategory.description%TYPE,
    P_importance IN ChangeCategory.importance%TYPE,
    P_creationApprovalDate IN ChangeCategory.creationApprovalDate%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
BEGIN
    inputParams:= 'Description: '||P_description||' - Importance: '||TO_CHAR(P_importance)||' - Creation Approval Date: '||TO_CHAR(P_creationApprovalDate, 'DD-MM-YYYY');

    INSERT INTO ChangeCategory (description, importance, creationDate, creationApprovalDate, deleteApprovalDate) 
        VALUES (P_description, P_importance, SYSDATE, P_creationApprovalDate, NULL);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addChangeCategory', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addChangeCategory', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento deleteChangeCategory
 * 
 * Descripción:
 *   Este procedimiento se utiliza para marcar una categoría de cambio como eliminada lógicamente en la tabla 'ChangeCategory'.
 *   Establece 'deleteApprovalDate' en la fecha actual, indicando que la categoría ha sido aprobada para eliminar.
 *   Registra cada operación en la tabla 'LogProcedure'.
 * 
 * Parámetros:
 *   P_categoryID IN ChangeCategory.IDCategory%TYPE
 *     - Descripción: ID de la categoría de cambio.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL y debe existir en la tabla 'ChangeCategory'.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Actualiza 'deleteApprovalDate' en la tabla 'ChangeCategory' para la categoría dada, marcándola como eliminada lógicamente.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE deleteChangeCategory(
    P_categoryID IN ChangeCategory.IDCategory%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
BEGIN
    inputParams:= 'CategoryID: '||TO_CHAR(P_categoryID);

    UPDATE ChangeCategory SET deleteApprovalDate = SYSDATE WHERE IDCategory = P_categoryID;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'deleteChangeCategory', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'deleteChangeCategory', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addChange
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'Change'.
 *   Registra cada operación en la tabla 'LogProcedure'.
 *   'requestDate' siempre se establece en la fecha actual y 'status' siempre en 'in_process'.
 *   Tras añadir un cambio, incrementa 'changesInApproval' en la tabla 'RE_Year' para el año actual.
 * 
 * Parámetros:
 *   P_description IN Change.description%TYPE
 *     - Descripción: Descripción del cambio.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_changeCategoryID IN Change.changeCategoryID%TYPE
 *     - Descripción: ID de la categoría de cambio.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla ChangeCategory.
 * 
 *   P_applicationID IN Change.applicationID%TYPE
 *     - Descripción: ID de la aplicación asociada.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla Application.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'Change' con 'requestDate' actual y 'status' en 'in_process'.
 *   2. Incrementa 'changesInApproval' en 'RE_Year' para el año actual.
 *   3. Registra la operación en la tabla 'LogProcedure'.
 *   4. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error.
 */
CREATE OR REPLACE PROCEDURE addChange(
    P_description IN Change.description%TYPE,
    P_changeCategoryID IN Change.changeCategoryID%TYPE,
    P_applicationID IN Change.applicationID%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    currentYear INT;
BEGIN
    inputParams:= 'Description: '||P_description||' - Change Category ID: '||TO_CHAR(P_changeCategoryID)||' - Application ID: '||TO_CHAR(P_applicationID);

    SELECT EXTRACT(YEAR FROM SYSDATE) INTO currentYear FROM DUAL;

    INSERT INTO Change (description, requestDate, status, changeCategoryID, applicationID) 
        VALUES (P_description, SYSDATE, 'in_process', P_changeCategoryID, P_applicationID);

    UPDATE RE_Year SET changesInApproval = changesInApproval + 1 WHERE yearData = currentYear;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addChange', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addChange', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento rejectChange
 * 
 * Descripción:
 *   Actualiza el estado de un registro en 'Change' a 'rejected' solo si el registro existe y su estado es 'in_process'.
 *   Resta 1 a 'changesInApproval' y suma 1 a 'notApprovedChanges' en 'RE_Year' para el año actual después de rechazar el cambio.
 *   Registra cada operación en la tabla 'LogProcedure'.
 * 
 * Parámetros:
 *   P_changeID IN Change.IDChange%TYPE
 *     - ID del cambio.
 *     - No puede ser NULL, debe existir en 'Change' y estar en estado 'in_process'.
 * 
 *   RSP OUT VARCHAR2
 *     - Indica el resultado de la operación ('OK' para éxito, 'ERROR' para fallos).
 * 
 * Funcionamiento:
 *   1. Verifica la existencia del cambio y su estado.
 *   2. Actualiza 'status' en 'Change' a 'rejected'.
 *   3. Actualiza 'RE_Year'.
 *   4. Registra la operación en 'LogProcedure'.
 *   5. Maneja excepciones.
 * 
 * Excepciones:
 *   - CHANGE_NOT_FOUND_EXCEPTION: Se lanza si el cambio no existe o no está en estado 'in_process'.
 *   - OTHERS: Captura otros errores.
 */
CREATE OR REPLACE PROCEDURE rejectChange(
    P_changeID IN Change.IDChange%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    currentYear INT;
    changeStatus VARCHAR2(255);
    CHANGE_NOT_FOUND_EXCEPTION EXCEPTION;
BEGIN
    inputParams:= 'Change ID: '||TO_CHAR(P_changeID);

    BEGIN
        SELECT status INTO changeStatus FROM Change WHERE IDChange = P_changeID;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE CHANGE_NOT_FOUND_EXCEPTION;
    END;

    IF changeStatus != 'in_process' THEN
        RAISE CHANGE_NOT_FOUND_EXCEPTION;
    END IF;

    SELECT EXTRACT(YEAR FROM SYSDATE) INTO currentYear FROM DUAL;

    UPDATE Change SET status = 'rejected' WHERE IDChange = P_changeID;

    UPDATE RE_Year SET changesInApproval = changesInApproval - 1, notApprovedChanges = notApprovedChanges + 1 
    WHERE yearData = currentYear;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'rejectChange', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN CHANGE_NOT_FOUND_EXCEPTION THEN
        errorNumber:= SQLCODE;
        RSP := 'ERROR - Change not found or not in process';
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'rejectChange', inputParams, RSP);
        COMMIT;
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'rejectChange', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento approveChange
 * 
 * Descripción:
 *   Actualiza el estado de un registro en 'Change' a 'approved' solo si el registro existe y su estado es 'in_process'.
 *   Añade un registro en 'ChangeExecution' con valores nulos para 'executionDate' y 'result'.
 *   Calcula y actualiza el técnico con más 'ChangeExecution' sin resultado en 'RE_Year'.
 *   Actualiza 'RE_Year', restando 1 a 'changesInApproval' y sumando 1 a 'GCABApprovedChanges' para el año actual.
 *   Registra cada operación en la tabla 'LogProcedure'.
 * 
 * Parámetros:
 *   P_changeID IN Change.IDChange%TYPE
 *     - ID del cambio.
 *     - No puede ser NULL, debe existir en 'Change' y estar en estado 'in_process'.
 * 
 *   RSP OUT VARCHAR2
 *     - Indica el resultado de la operación ('OK' para éxito, 'ERROR' para fallos).
 * 
 * Funcionamiento:
 *   1. Verifica la existencia del cambio y su estado.
 *   2. Actualiza 'status' en 'Change' a 'approved'.
 *   3. Inserta en 'ChangeExecution'.
 *   4. Calcula y actualiza el técnico con más 'ChangeExecution' sin resultado.
 *   5. Actualiza 'RE_Year'.
 *   6. Registra la operación en 'LogProcedure'.
 *   7. Maneja excepciones.
 * 
 * Excepciones:
 *   - CHANGE_NOT_FOUND_EXCEPTION: Se lanza si el cambio no existe o no está en estado 'in_process'.
 *   - OTHERS: Captura otros errores.
 */
CREATE OR REPLACE PROCEDURE approveChange(
    P_changeID IN Change.IDChange%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    currentYear INT;
    changeExists INT;
    changeStatus VARCHAR2(255);
    mostOpenChangesTechID INT;
    mostOpenChangesTechName VARCHAR2(255);
    mostOpenChangesTechContact VARCHAR2(255);
    CHANGE_NOT_FOUND_EXCEPTION EXCEPTION;
BEGIN
    inputParams:= 'Change ID: '||TO_CHAR(P_changeID);

    SELECT status INTO changeStatus FROM Change WHERE IDChange = P_changeID;
    IF changeStatus != 'in_process' THEN
        RAISE CHANGE_NOT_FOUND_EXCEPTION;
    END IF;

    SELECT EXTRACT(YEAR FROM SYSDATE) INTO currentYear FROM DUAL;

    UPDATE Change SET status = 'approved' WHERE IDChange = P_changeID;

    INSERT INTO ChangeExecution (executionDate, result, changeID) 
        VALUES (NULL, NULL, P_changeID);

    -- Calcular el técnico con más cambios en curso
    SELECT technicalContact INTO mostOpenChangesTechID FROM (
        SELECT technicalContact, COUNT(*) AS openChanges
        FROM Change JOIN Application ON Change.applicationID = Application.IDApplication
        JOIN ChangeExecution ON Change.IDChange = ChangeExecution.changeID
        WHERE ChangeExecution.result IS NULL
        GROUP BY technicalContact
        ORDER BY openChanges DESC
        FETCH FIRST ROW ONLY
    );

    SELECT name, contact INTO mostOpenChangesTechName, mostOpenChangesTechContact 
    FROM Approver 
    WHERE IDApprover = mostOpenChangesTechID;

    -- Actualizar RE_Year con el técnico con más cambios en curso
    UPDATE RE_Year 
    SET maxOpenChangesTechnicalManager = mostOpenChangesTechName || '-' || mostOpenChangesTechContact, 
        changesInApproval = changesInApproval - 1, 
        GCABApprovedChanges = GCABApprovedChanges + 1 
    WHERE yearData = currentYear;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'approveChange', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN CHANGE_NOT_FOUND_EXCEPTION THEN
        errorNumber:= SQLCODE;
        RSP := 'ERROR - Change not found or not in process';
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'approveChange', inputParams, RSP);
        COMMIT;
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'approveChange', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento vote
 * 
 * Descripción:
 *   Añade un registro en la tabla 'Approval' para registrar el voto.
 *   Verifica si el approverID está disponible, autorizado para la aplicación especificada, y tiene capacidad de voto.
 *   El voto no puede ser nulo y la fecha de aprobación se establece automáticamente en la fecha actual.
 *   El sustituto puede votar solo si existe un registro en 'ApplicationApprovers' indicando su autorización.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja errores durante la inserción.
 * 
 * Parámetros:
 *   P_approverID IN Approval.approverID%TYPE
 *     - ID del aprobador.
 *     - No puede ser NULL y debe existir en la tabla 'Approver'.
 * 
 *   P_changeID IN Approval.changeID%TYPE
 *     - ID del cambio.
 *     - No puede ser NULL y debe existir en la tabla 'Change'.
 * 
 *   P_vote IN Approval.vote%TYPE
 *     - Voto del aprobador (0 para rechazo, 1 para aprobación).
 *     - Debe ser 0 o 1 y no puede ser NULL.
 * 
 *   P_comments IN Approval.comments%TYPE
 *     - Comentarios adicionales.
 * 
 *   RSP OUT VARCHAR2
 *     - Indica el resultado de la operación ('OK' para éxito, 'ERROR' para fallos).
 * 
 * Funcionamiento:
 *   1. Verifica la disponibilidad, autorización y capacidad de voto del approverID.
 *   2. Inserta el registro en 'Approval'.
 *   3. Registra la operación en 'LogProcedure'.
 *   4. Maneja excepciones para votos nulos y otros errores.
 * 
 * Excepciones:
 *   - VOTE_CANNOT_BE_NULL: Se lanza cuando el voto es NULL.
 *   - NOT_AUTHORIZED_EXCEPTION: Se lanza cuando el approver no está autorizado o no tiene derechos de voto.
 *   - OTHERS: Captura otros errores.
 */
CREATE OR REPLACE PROCEDURE vote(
    P_approverID IN Approval.approverID%TYPE,
    P_changeID IN Approval.changeID%TYPE,
    P_vote IN Approval.vote%TYPE,
    P_comments IN Approval.comments%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    VOTE_CANNOT_BE_NULL EXCEPTION;
    NOT_AUTHORIZED_EXCEPTION EXCEPTION;
    hasVote NUMBER(1);
    isAvailable NUMBER(1);
    isAuthorized NUMBER(1);

BEGIN
    inputParams:= 'ApproverID: '||P_approverID||' - ChangeID: '||P_changeID||' - Vote: '||P_vote||' - Comments: '||P_comments;

    IF P_vote IS NULL THEN
        RAISE VOTE_CANNOT_BE_NULL;
    END IF;

    -- Verifica disponibilidad, autorización y capacidad de voto del approver
    SELECT Approver.isAvailable, ApproverRole.hasVote INTO isAvailable, hasVote 
    FROM Approver 
    JOIN ApproverRole ON Approver.approverRoleID = ApproverRole.IDRole 
    WHERE Approver.IDApprover = P_approverID;

    IF isAvailable = 1 THEN
        SELECT COUNT(*) INTO isAuthorized 
        FROM ApplicationApprovers 
        WHERE approverID = P_approverID AND applicationID = (SELECT applicationID FROM Change WHERE IDChange = P_changeID);

        IF isAuthorized = 0 OR hasVote = 0 THEN
            RAISE NOT_AUTHORIZED_EXCEPTION;
        END IF;
    ELSE
        RAISE NOT_AUTHORIZED_EXCEPTION;
    END IF;

    INSERT INTO Approval (approverID, changeID, approvalDate, vote, comments) 
        VALUES (P_approverID, P_changeID, SYSDATE, P_vote, P_comments);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'vote', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN VOTE_CANNOT_BE_NULL THEN
        errorNumber:= SQLCODE;
        RSP:= 'ERROR - Vote cannot be NULL';
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'vote', inputParams, RSP);
        COMMIT;
    
    WHEN NOT_AUTHORIZED_EXCEPTION THEN
        errorNumber:= SQLCODE;
        RSP:= 'ERROR - Approver not authorized or has no voting rights';
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'vote', inputParams, RSP);
        COMMIT;

    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'vote', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addComment
 * 
 * Descripción:
 *   Este procedimiento añade un registro en la tabla 'Approval' enfocado en los comentarios, sin registrar un voto.
 *   'approvalDate' se establece automáticamente en la fecha actual y 'vote' es siempre NULL.
 *   Verifica si el approverID está disponible y habilitado para la aplicación especificada.
 *   El sustituto puede comentar solo si existe un registro en 'ApplicationApprovers' indicando su autorización.
 * 
 * Parámetros:
 *   P_approverID IN Approval.approverID%TYPE
 *     - ID del aprobador.
 *     - No puede ser NULL y debe existir en la tabla 'Approver'.
 * 
 *   P_changeID IN Approval.changeID%TYPE
 *     - ID del cambio.
 *     - No puede ser NULL y debe existir en la tabla 'Change'.
 * 
 *   P_comments IN Approval.comments%TYPE
 *     - Comentarios adicionales.
 *     - No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Indica el resultado de la operación ('OK' para éxito, 'ERROR' para fallos).
 * 
 * Funcionamiento:
 *   1. Verifica si el approverID está disponible y habilitado para la aplicación.
 *   2. Inserta el registro en 'Approval' con la fecha actual y 'vote' en NULL.
 *   3. Registra la operación en 'LogProcedure'.
 *   4. Maneja excepciones para autorizaciones faltantes y otros errores.
 * 
 * Excepciones:
 *   - COMMENT_CANNOT_BE_NULL: Se lanza cuando los comentarios son NULL.
 *   - NOT_AUTHORIZED_EXCEPTION: Se lanza cuando el approver no está autorizado o no disponible.
 *   - OTHERS: Captura otros errores.
 */
CREATE OR REPLACE PROCEDURE addComment(
    P_approverID IN Approval.approverID%TYPE,
    P_changeID IN Approval.changeID%TYPE,
    P_comments IN Approval.comments%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    COMMENT_CANNOT_BE_NULL EXCEPTION;
    NOT_AUTHORIZED_EXCEPTION EXCEPTION;
    isAvailable NUMBER(1);
    isAuthorized NUMBER(1);

BEGIN
    inputParams:= 'ApproverID: '||P_approverID||' - ChangeID: '||P_changeID||' - Comments: '||P_comments;

    IF P_comments IS NULL THEN
        RAISE COMMENT_CANNOT_BE_NULL;
    END IF;

    -- Verifica disponibilidad y autorización del approver
    SELECT Approver.isAvailable INTO isAvailable 
    FROM Approver 
    WHERE Approver.IDApprover = P_approverID;

    IF isAvailable = 1 THEN
        SELECT COUNT(*) INTO isAuthorized 
        FROM ApplicationApprovers 
        WHERE approverID = P_approverID AND applicationID = (
            SELECT applicationID FROM Change WHERE IDChange = P_changeID
            );

        IF isAuthorized = 0 THEN
            RAISE NOT_AUTHORIZED_EXCEPTION;
        END IF;
    ELSE
        RAISE NOT_AUTHORIZED_EXCEPTION;
    END IF;

    INSERT INTO Approval (approverID, changeID, approvalDate, vote, comments) 
        VALUES (P_approverID, P_changeID, SYSDATE, NULL, P_comments);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addComment', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN COMMENT_CANNOT_BE_NULL THEN
        errorNumber:= SQLCODE;
        RSP:= 'ERROR - Comments cannot be NULL';
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addComment', inputParams, RSP);
        COMMIT;

    WHEN NOT_AUTHORIZED_EXCEPTION THEN
        errorNumber:= SQLCODE;
        RSP:= 'ERROR - Approver not authorized or not available';
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addComment', inputParams, RSP);
        COMMIT;

    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addComment', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento executeChange
 * 
 * Descripción:
 *   Este procedimiento actualiza un registro en la tabla 'ChangeExecution' y, 
 *   si es necesario, crea un nuevo registro en 'ActionPlan'.
 *   Dependiendo del resultado de la ejecución del cambio, realiza varias actualizaciones estadísticas:
 *     - Si el resultado es 'incidents' o 'rescheduled', incrementa 'incorrectExecution' en 'RE_Year' para el año actual.
 *     - Si el resultado es 'correct', calcula el técnico con más cambios correctos ejecutados de cualquier aplicación.
 *     - Si el resultado es 'rescheduled', calcula el porcentaje de cambios replanificados por zona geográfica.
 *     - Cada vez que se crea un 'ActionPlan', aumenta en 1 'openActionPlans' en 'RE_Year' para el año actual.
 *     - Tras actualizar un 'ChangeExecution', calcula el técnico con más cambios en curso (sin 'result') y actualiza 'maxOpenChangesTechnicalManager' en 'RE_Year'.
 * 
 * Parámetros:
 *   P_executionID IN ChangeExecution.IDExecution%TYPE
 *     - Descripción: ID de la ejecución del cambio.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla 'ChangeExecution'.
 * 
 *   P_executionDate IN ChangeExecution.executionDate%TYPE
 *     - Descripción: Fecha de ejecución del cambio.
 *     - Tipo: Fecha (DATE).
 * 
 *   P_result IN ChangeExecution.result%TYPE
 *     - Descripción: Resultado de la ejecución del cambio ('correct', 'incidents', 'rescheduled').
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: Debe ser uno de los valores especificados.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Actualiza el registro de 'ChangeExecution' con la fecha y resultado proporcionados.
 *   2. Si el resultado es 'incidents' o 'rescheduled', crea un nuevo 'ActionPlan' y aumenta 'incorrectExecution'.
 *   3. Si el resultado es 'correct', calcula el técnico con más cambios correctos ejecutados de cualquier aplicación.
 *   4. Si el resultado es 'rescheduled', calcula el porcentaje de cambios replanificados por zona geográfica.
 *   5. Independientemente del resultado, calcula el técnico con más cambios en curso y actualiza 'maxOpenChangesTechnicalManager'.
 *   6. Registra la operación en la tabla 'LogProcedure'.
 *   7. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE executeChange(
    P_executionID IN ChangeExecution.IDExecution%TYPE,
    P_executionDate IN ChangeExecution.executionDate%TYPE,
    P_result IN ChangeExecution.result%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    currentYear INT;
    topTechnicalContactID INT;
    topTechContactName VARCHAR2(255);
    topTechContactContact VARCHAR2(255);
    mostOpenChangesTechID INT;
    mostOpenChangesTechName VARCHAR2(255);
    mostOpenChangesTechContact VARCHAR2(255);
    totalOpenChanges INT;
    changeDescription VARCHAR2(1000);
    zoneName VARCHAR2(255);
    percentageRescheduled NUMBER;
    totalChanges INT;
    rescheduledChanges INT;

BEGIN
    inputParams:= 'Execution ID: '||P_executionID||' - Execution Date: '||P_executionDate||' - Result: '||P_result;

    SELECT EXTRACT(YEAR FROM SYSDATE) INTO currentYear FROM DUAL;

    UPDATE ChangeExecution 
    SET executionDate = P_executionDate, result = P_result 
    WHERE IDExecution = P_executionID;

    IF P_result = 'correct' THEN
        BEGIN
            -- Técnico con más cambios correctos ejecutados
            SELECT technicalContact INTO topTechnicalContactID FROM (
                SELECT technicalContact, COUNT(*) AS correctChanges
                FROM Change JOIN Application ON Change.applicationID = Application.IDApplication
                    JOIN ChangeExecution ON Change.IDChange = ChangeExecution.changeID
                WHERE Change.status = 'approved' AND ChangeExecution.result = 'correct'
                GROUP BY technicalContact
                ORDER BY correctChanges DESC
                FETCH FIRST ROW ONLY
            );

            SELECT name, contact INTO topTechContactName, topTechContactContact 
            FROM Approver 
            WHERE IDApprover = topTechnicalContactID;

            -- Actualiza RE_Year con el nombre y contacto
            UPDATE RE_Year 
            SET maxCorrectChangesTechnicalManager = topTechContactName || ' | ' || topTechContactContact 
            WHERE yearData = currentYear;

        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                NULL; -- No existen datos todavía
        END;

    ELSIF P_result IN ('incidents', 'rescheduled') THEN
        SELECT description INTO changeDescription FROM Change WHERE IDChange = (
            SELECT changeID FROM ChangeExecution WHERE IDExecution = P_executionID
        );

        INSERT INTO ActionPlan (description, isClosed, changeExecutionID) VALUES (changeDescription || '-' || P_result, 0, P_executionID);

        UPDATE RE_Year SET incorrectExecution = incorrectExecution + 1, openActionPlans = openActionPlans + 1 WHERE yearData = currentYear;
    END IF;

    IF P_result = 'rescheduled' THEN
        BEGIN
            -- Obtener la zona geográfica del cambio
            SELECT name INTO zoneName FROM GeographicZone 
            WHERE IDZone = (
                SELECT geographicZoneID FROM Application 
                WHERE IDApplication = (
                    SELECT applicationID FROM Change WHERE IDChange = (
                        SELECT changeID FROM ChangeExecution WHERE IDExecution = P_executionID
                    )
                )
            );

            -- Calcular el porcentaje de cambios replanificados
            SELECT COUNT(*) INTO totalChanges 
            FROM Change JOIN Application ON Change.applicationID = Application.IDApplication
            WHERE Application.geographicZoneID = (
                SELECT geographicZoneID FROM GeographicZone WHERE name = zoneName
            );

            SELECT COUNT(*) INTO rescheduledChanges 
            FROM Change JOIN ChangeExecution ON Change.IDChange = ChangeExecution.changeID
            WHERE ChangeExecution.result = 'rescheduled' AND Change.applicationID IN (
                SELECT IDApplication FROM Application WHERE geographicZoneID = (
                    SELECT IDZone FROM GeographicZone WHERE name = zoneName
                )
            );

            percentageRescheduled := ROUND((rescheduledChanges / totalChanges) * 100, 2);

            -- Actualizar RE_Geographic
            UPDATE RE_Geographic 
            SET percentajeRescheduledChanges = percentageRescheduled 
            WHERE geographicZone = zoneName;

        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                NULL; -- No se encontraron datos
        END;
    END IF;

    BEGIN
        -- Calcular técnico con más cambios en curso
        SELECT technicalContact INTO mostOpenChangesTechID FROM (
            SELECT technicalContact, COUNT(*) AS openChanges
            FROM Change JOIN Application ON Change.applicationID = Application.IDApplication
            JOIN ChangeExecution ON Change.IDChange = ChangeExecution.changeID
            WHERE ChangeExecution.result IS NULL
            GROUP BY technicalContact
            ORDER BY openChanges DESC
            FETCH FIRST ROW ONLY
        );

        SELECT name, contact INTO mostOpenChangesTechName, mostOpenChangesTechContact 
        FROM Approver 
        WHERE IDApprover = mostOpenChangesTechID;

        -- Actualizar RE_Year con el técnico con más cambios en curso
        UPDATE RE_Year 
        SET maxOpenChangesTechnicalManager = mostOpenChangesTechName || '-' || mostOpenChangesTechContact 
        WHERE yearData = currentYear;
    EXCEPTION
            WHEN NO_DATA_FOUND THEN
                NULL; -- No se encontraron datos
        END;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'executeChange', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'executeChange', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento addAction
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'Action'.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja errores durante la inserción.
 *   'doneDate' se establece en NULL por defecto al añadir un registro.
 * 
 * Parámetros:
 *   P_description IN Action.description%TYPE
 *     - Descripción: Descripción de la acción.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_deliveryDate IN Action.deliveryDate%TYPE
 *     - Descripción: Fecha de entrega prevista para la acción.
 *     - Tipo: Fecha (DATE).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_actionPlanID IN Action.actionPlanID%TYPE
 *     - Descripción: ID del plan de acción asociado.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'Action' con 'doneDate' en NULL.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE addAction(
    P_description IN Action.description%TYPE,
    P_deliveryDate IN Action.deliveryDate%TYPE,
    P_actionPlanID IN Action.actionPlanID%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);

BEGIN
    inputParams:= 'Description: '||P_description||' - DeliveryDate: '||TO_CHAR(P_deliveryDate, 'YYYY-MM-DD')||' - ActionPlanID: '||TO_CHAR(P_actionPlanID);

    INSERT INTO Action (description, deliveryDate, doneDate, actionPlanID) 
        VALUES (P_description, P_deliveryDate, NULL, P_actionPlanID);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addAction', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addAction', inputParams, RSP);
        COMMIT;
END;
/

/**
 * Procedimiento closeAction
 * 
 * Descripción:
 *   Este procedimiento actualiza la fecha de finalización (doneDate) de una acción específica en la tabla 'Action',
 *   y recalcula el porcentaje de acciones cerradas a tiempo en 'RE_Year'.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja errores durante la actualización.
 * 
 * Parámetros:
 *   P_actionID IN Action.IDAction%TYPE
 *     - Descripción: ID de la acción a actualizar.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla 'Action'.
 * 
 *   P_doneDate IN Action.doneDate%TYPE
 *     - Descripción: Fecha en que se completó la acción.
 *     - Tipo: Fecha (DATE).
 *     - Restricciones: No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Actualiza 'doneDate' de la acción especificada y recalcula el porcentaje de acciones completadas a tiempo.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - ACTION_ALREADY_CLOSED: Se lanza cuando la acción ya está cerrada (doneDate no es NULL).
 *   - INVALID_DONE_DATE: Se lanza cuando la doneDate proporcionada es NULL.
 *   - OTHERS: Captura cualquier otro tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE closeAction(
    P_actionID IN Action.IDAction%TYPE,
    P_doneDate IN Action.doneDate%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    currentDoneDate DATE;
    deliveryDate DATE;
    totalActions INT;
    actionsInTime INT;
    currentYear INT;
    ACTION_ALREADY_CLOSED EXCEPTION;
    INVALID_DONE_DATE EXCEPTION;

BEGIN
    inputParams:= 'Action ID: '||P_actionID||' - DoneDate: '||NVL(TO_CHAR(P_doneDate, 'YYYY-MM-DD'), 'NULL');

    IF P_doneDate IS NULL THEN
        RAISE INVALID_DONE_DATE;
    END IF;

    SELECT doneDate, deliveryDate INTO currentDoneDate, deliveryDate FROM Action WHERE IDAction = P_actionID;
    IF currentDoneDate IS NOT NULL THEN
        RAISE ACTION_ALREADY_CLOSED;
    END IF;

    UPDATE Action SET doneDate = P_doneDate WHERE IDAction = P_actionID;

    SELECT EXTRACT(YEAR FROM SYSDATE) INTO currentYear FROM DUAL;

    -- Calcular y actualizar el porcentaje de acciones completadas a tiempo
    SELECT COUNT(*) INTO totalActions FROM Action 
    WHERE EXTRACT(YEAR FROM deliveryDate) = currentYear;

    IF totalActions > 0 THEN
        SELECT COUNT(*) INTO actionsInTime FROM Action 
        WHERE EXTRACT(YEAR FROM deliveryDate) = currentYear AND (doneDate <= deliveryDate OR (doneDate IS NULL AND SYSDATE <= deliveryDate));

        UPDATE RE_Year 
        SET doneActionsInTimePercentaje = ROUND((actionsInTime / totalActions) * 100, 2) 
        WHERE yearData = currentYear;
    END IF;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'closeAction', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN ACTION_ALREADY_CLOSED THEN
        RSP:= 'ERROR - Action already closed';
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'closeAction', inputParams, RSP);
        COMMIT;

    WHEN INVALID_DONE_DATE THEN
        RSP:= 'ERROR - Invalid doneDate: cannot be NULL';
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'closeAction', inputParams, RSP);
        COMMIT;

    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'closeAction', inputParams, RSP);
        COMMIT;
END;
/


/**
 * Procedimiento closeActionPlan
 * 
 * Descripción:
 *   Marca un 'ActionPlan' específico como cerrado y genera un nuevo registro en 'ChangeExecution'.
 *   Calcula y actualiza el técnico con más 'ChangeExecution' sin resultado en 'RE_Year'.
 *   Actualiza 'RE_Year' restando 1 a 'openActionPlans' y actualizando 'maxOpenChangesTechnicalManager'.
 *   Registra cada operación en la tabla 'LogProcedure'.
 * 
 * Parámetros:
 *   P_actionPlanID IN ActionPlan.IDActionPlan%TYPE
 *     - Descripción: ID del plan de acción.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL, debe existir en la tabla 'ActionPlan'.
 * 
 *   P_newDescription IN ActionPlan.description%TYPE
 *     - Descripción: Nueva descripción para el plan de acción.
 *     - Tipo: Cadena de texto (VARCHAR2(1000)).
 *     - Restricciones: Puede ser NULL. Si es NULL, la descripción no se actualizará.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Actualiza el estado de 'isClosed' a 1 en 'ActionPlan' y, opcionalmente, su descripción.
 *   2. Crea un nuevo registro en 'ChangeExecution' con 'executionDate' y 'result' como NULL.
 *   3. Calcula y actualiza el técnico con más cambios en curso en 'RE_Year'.
 *   4. Actualiza 'RE_Year', restando 1 de 'openActionPlans'.
 *   5. Registra la operación en la tabla 'LogProcedure'.
 *   6. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE closeActionPlan(
    P_actionPlanID IN ActionPlan.IDActionPlan%TYPE,
    P_newDescription IN ActionPlan.description%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);
    changeExecutionID INT;
    relatedChangeExecutionID INT;
    mostOpenChangesTechID INT;
    mostOpenChangesTechName VARCHAR2(255);
    mostOpenChangesTechContact VARCHAR2(255);
    currentYear INT;

BEGIN
    inputParams:= 'ActionPlan ID: '||P_actionPlanID||', New Description: '||NVL(P_newDescription, 'No Change');

    UPDATE ActionPlan SET isClosed = 1, description = P_newDescription WHERE IDActionPlan = P_actionPlanID;

    SELECT changeExecutionID INTO relatedChangeExecutionID FROM ActionPlan WHERE IDActionPlan = P_actionPlanID;

    INSERT INTO ChangeExecution (executionDate, result, changeID) 
        VALUES (NULL, NULL, (SELECT changeID FROM ChangeExecution WHERE IDExecution = relatedChangeExecutionID));

    -- Calcular el técnico con más cambios en curso
    SELECT technicalContact INTO mostOpenChangesTechID FROM (
        SELECT technicalContact, COUNT(*) AS openChanges
        FROM Change JOIN Application ON Change.applicationID = Application.IDApplication
        JOIN ChangeExecution ON Change.IDChange = ChangeExecution.changeID
        WHERE ChangeExecution.result IS NULL
        GROUP BY technicalContact
        ORDER BY openChanges DESC
        FETCH FIRST ROW ONLY
    );

    SELECT name, contact INTO mostOpenChangesTechName, mostOpenChangesTechContact 
    FROM Approver 
    WHERE IDApprover = mostOpenChangesTechID;

    SELECT EXTRACT(YEAR FROM SYSDATE) INTO currentYear FROM DUAL;

    -- Actualizar RE_Year con el técnico con más cambios en curso y disminuir los planes de acción abiertos
    UPDATE RE_Year 
    SET maxOpenChangesTechnicalManager = mostOpenChangesTechName || '-' || mostOpenChangesTechContact,
        openActionPlans = openActionPlans - 1
    WHERE yearData = currentYear;

    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'closeActionPlan', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'closeActionPlan', inputParams, RSP);
        COMMIT;
END;
/




/**
 * Procedimiento addProcessAudit
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'ProcessAudit' con la fecha actual.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja errores durante la inserción.
 * 
 * Parámetros:
 *   P_observations IN ProcessAudit.observations%TYPE
 *     - Descripción: Observaciones de la auditoría.
 *     - Tipo: Cadena de texto (VARCHAR2).
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'ProcessAudit' con la fecha actual.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE addProcessAudit(
    P_observations IN ProcessAudit.observations%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);

BEGIN
    inputParams:= 'AuditDate: '||TO_CHAR(SYSDATE, 'YYYY-MM-DD')||' - Observations: '||P_observations;

    INSERT INTO ProcessAudit (auditDate, observations) 
        VALUES (SYSDATE, P_observations);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addProcessAudit', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addProcessAudit', inputParams, RSP);
        COMMIT;
END;
/




/**
 * Procedimiento addAuditComment
 * 
 * Descripción:
 *   Este procedimiento se utiliza para añadir un nuevo registro en la tabla 'AuditComment'.
 *   Registra cada operación en la tabla 'LogProcedure' y maneja errores durante la inserción.
 * 
 * Parámetros:
 *   P_nonCompliances IN AuditComment.nonCompliances%TYPE
 *     - Descripción: Incumplimiento observado en la auditoría.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Restricciones: No puede ser NULL.
 * 
 *   P_observations IN AuditComment.observations%TYPE
 *     - Descripción: Observaciones adicionales de la auditoría.
 *     - Tipo: Cadena de texto (VARCHAR2).
 * 
 *   P_auditID IN AuditComment.auditID%TYPE
 *     - Descripción: ID de la auditoría asociada.
 *     - Tipo: Entero (INT).
 *     - Restricciones: No puede ser NULL.
 * 
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Inserta el nuevo registro en la tabla 'AuditComment'.
 *   2. Registra la operación en la tabla 'LogProcedure'.
 *   3. Maneja cualquier excepción que ocurra durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE addAuditComment(
    P_nonCompliances IN AuditComment.nonCompliances%TYPE,
    P_observations IN AuditComment.observations%TYPE,
    P_auditID IN AuditComment.auditID%TYPE,
    RSP OUT VARCHAR2) 
IS
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(1000);

BEGIN
    inputParams:= 'NonCompliances: '||P_nonCompliances||' - Observations: '||P_observations||' - AuditID: '||TO_CHAR(P_auditID);

    INSERT INTO AuditComment (nonCompliances, observations, auditID) 
        VALUES (P_nonCompliances, P_observations, P_auditID);
    RSP:= 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'addAuditComment', inputParams, RSP);
    COMMIT;

EXCEPTION  
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
            VALUES(SYSDATE, USER, 'addAuditComment', inputParams, RSP);
        COMMIT;
END;
/





-- Procedimientos RE

/**
 * Procedimiento updateLastSixMonthsAuditCount
 * 
 * Descripción:
 *   Este procedimiento cuenta los registros AuditComment generados en los últimos 6 meses
 *   y los inserta en la tabla RE_LastSixMonths.
 * 
 * Parámetros:
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 * 
 * Funcionamiento:
 *   1. Cuenta los registros AuditComment de los últimos 6 meses.
 *   2. Inserta el conteo en la tabla RE_LastSixMonths.
 *   3. Registra la operación en la tabla 'LogProcedure'.
 *   4. Maneja excepciones para errores durante la ejecución.
 * 
 * Excepciones:
 *   - OTHERS: Captura cualquier otro tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE updateLastSixMonthsAuditCount(RSP OUT VARCHAR2) 
IS
    v_count INT;
    v_today DATE := TRUNC(SYSDATE);
    inputParams VARCHAR2(1000) := 'Execution Date: ' || TO_CHAR(v_today, 'YYYY-MM-DD');
    errorNumber INTEGER;
    errorType VARCHAR2(1000);

BEGIN
    -- Contar registros AuditComment de los últimos 6 meses
    SELECT COUNT(*)
    INTO v_count
    FROM AuditComment AC
    JOIN ProcessAudit PA ON AC.auditID = PA.IDAudit
    WHERE PA.auditDate >= ADD_MONTHS(v_today, -6);

    -- Insertar el conteo en RE_LastSixMonths
    INSERT INTO RE_LastSixMonths (lastDate, numNonCompliances)
    VALUES (v_today, v_count);

    RSP := 'OK';
    COMMIT;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
    VALUES (SYSDATE, USER, 'updateLastSixMonthsAuditCount', inputParams, RSP);
    COMMIT;

EXCEPTION

    WHEN OTHERS THEN
        errorNumber := SQLCODE;
        errorType := SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - ' || TO_CHAR(errorNumber) || ' - ' || errorType;
        ROLLBACK;
        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES (SYSDATE, USER, 'updateLastSixMonthsAuditCount', inputParams, RSP);
        COMMIT;
END;
/




/**
 * Procedimiento checkAndCreateCurrentYearREYear
 *
 * Descripción:
 *   Este procedimiento verifica si existe un registro en la tabla 'RE_Year' para el año actual.
 *   Si no existe, crea un nuevo registro con el año actual y copia ciertos valores del año anterior.
 *   Además, registra cada operación en la tabla 'LogProcedure'.
 *
 * Parámetros:
 *   Ninguno.
 *
 *   RSP OUT VARCHAR2
 *     - Descripción: Parámetro de salida que indica el resultado de la operación.
 *     - Tipo: Cadena de texto (VARCHAR2).
 *     - Valores posibles: 'OK' para éxito, 'ERROR - [código de error] - [mensaje de error]' para fallos.
 *
 * Funcionamiento:
 *   1. Verifica la existencia de un registro para el año actual en la tabla 'RE_Year'.
 *   2. Si no existe, crea un nuevo registro con valores del año anterior para campos específicos.
 *   3. Registra la operación en la tabla 'LogProcedure'.
 *   4. Maneja cualquier excepción que ocurra durante la ejecución.
 *
 * Excepciones:
 *   - OTHERS: Captura cualquier tipo de error durante la ejecución del procedimiento.
 */
CREATE OR REPLACE PROCEDURE checkAndCreateCurrentYearREYear(RSP OUT VARCHAR2) 
IS
    currentYear INT;
    lastYear INT;
    countYear INT;
    lastYearData RE_Year%ROWTYPE;
    inputParams VARCHAR2(1000);
    errorNumber INTEGER;
    errorType VARCHAR2(255);
BEGIN
    SELECT EXTRACT(YEAR FROM SYSDATE) INTO currentYear FROM DUAL;
    lastYear := currentYear - 1;
    inputParams:= 'Year: '||currentYear;

    SELECT COUNT(*) INTO countYear FROM RE_Year WHERE yearData = currentYear;

    IF countYear = 0 THEN
        BEGIN
            SELECT * INTO lastYearData FROM RE_Year WHERE yearData = lastYear;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                lastYearData.changesInApproval := 0;
                lastYearData.maxCorrectChangesTechnicalManager := '';
                lastYearData.openActionPlans := 0;
                lastYearData.maxOpenChangesTechnicalManager := '';
                lastYearData.mostReplacedApprover := '';
        END;

        INSERT INTO RE_Year (yearData, changesInApproval, incorrectExecution, 
                             maxCorrectChangesTechnicalManager, doneActionsInTimePercentaje, 
                             GCABApprovedChanges, notApprovedChanges, openActionPlans, 
                             maxOpenChangesTechnicalManager, mostReplacedApprover)
        VALUES (currentYear, lastYearData.changesInApproval, 0, 
                lastYearData.maxCorrectChangesTechnicalManager, 0, 0, 0, 
                lastYearData.openActionPlans, lastYearData.maxOpenChangesTechnicalManager, 
                lastYearData.mostReplacedApprover);

        RSP:= 'OK';
    ELSE
        RSP:= 'ERROR - Year data already exists';
    END IF;

    INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
    VALUES(SYSDATE, USER, 'checkAndCreateCurrentYearREYear', inputParams, RSP);

    COMMIT;

EXCEPTION
    WHEN OTHERS THEN
        errorNumber:= SQLCODE;
        errorType:= SUBSTR(SQLERRM, 1, 255);
        RSP := 'ERROR - '||TO_CHAR(errorNumber)||' - '||errorType;
        ROLLBACK;

        INSERT INTO LogProcedure (logDateTime, logUser, procedureName, procedureParams, RSP) 
        VALUES(SYSDATE, USER, 'checkAndCreateCurrentYearREYear', inputParams, RSP);
END;
/


